using System;
using System.Windows.Forms;
using DicomObjects;

namespace LicenceManipulation
{
    public partial class Form1 : Form
    {
        private const string LogMachineIDInfo = "LogMachineIDInfo";
        private const string BitMode64 = "64 bit mode";
        private const string BitMode32 = "32 bit mode";
        private const string AuthorisationText = "XXXX-YYYY-ZZZZ";
        private const string AuthorisationString = "Enter your License Key in XXXX-YYYY-ZZZZ format";
        private const string CustomerInfo1 = "Enter the licensee's Name or ID etc.";
        private const string CustomerInfo2 = "Enter other information that can identify the license";
        private const string LicenseDetailText = "License Details";
        private const string DeleteMessage = "Are you really sure?";
        private const string DeleteCaption = "Delete Selected?";
        private const string DeactivationResultText = "Deactivation result";
        private const string MachineIdText = "MachineID = ";
        private const string AllowedMessageText = "Allowed Functions:";
        private const string AuthorisationIDText = "AuthorisationID:";
        private const string ExpiryDateText = "Expiry Date:";
        private const string IsRamLicenseText = "IsRamLicense:";
        private const string LicenseIdText = "License ID: ";
        private const string LicenseTokenText = "License Token:";
        private const string LicenseTypeText = "License Type:";

        public Form1()
        {
            InitializeComponent();            
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Text += $"        DicomObjects.v {DicomGlobal.FileVersion}";
            // this helps trouble-shoot license problems
            DicomGlobal.SetRegWord(LogMachineIDInfo, 1);

            // if there is a problem with the license activation, send us the log for checking
            // you should find the D.O.Net.xxx.log in the LicenceManipulation.exe folder 
            DicomGlobal.LogToFile(Environment.CurrentDirectory, 0x3F);

            FillBox();
        }

        private void FillBox()
        {
            licenseList.Items.Clear();

            mode.Text = Environment.Is64BitProcess ? BitMode64 : BitMode32;
            // DicomLicensne.MachineID is generated by DicomObjects to uniquely identifies each machine
            // MachineID might change due to hardware changes, but should not change after Windows update
            machineID.Text = MachineIdText + DicomLicense.MachineID.ToString();
                   
            // list all DicomObjects licenses found on the machine, including expired ones    
            foreach (DicomLicense license in DicomLicense.InstalledLicenses)
            {
                var item = new ListViewItem(license.LicenseType.ToString())
                {
                    Tag = license
                };
                item.SubItems.Add(license.Expiry.ToString("D"));
                item.SubItems.Add(license.LicenseID.ToString());
                item.SubItems.Add(license.AuthorisationID.ToString());
                item.ForeColor = (license.Expiry < DateTime.Now)?System.Drawing.Color.Red: System.Drawing.Color.Green;

                licenseList.Items.Add(item);
            }
        }

        private void GetLicenseBTN_Click(object sender, EventArgs e)
        {
            DicomLicense.Dialog.AuthorisationString.Text = AuthorisationText;
            NewTooltip().SetToolTip(DicomLicense.Dialog.AuthorisationString, AuthorisationString);
            NewTooltip().SetToolTip(DicomLicense.Dialog.CustomerInfo1, CustomerInfo1);
            NewTooltip().SetToolTip(DicomLicense.Dialog.CustomerInfo2, CustomerInfo2);
            DicomLicense.Dialog.ShowDialog();
            FillBox();
        }

        private ToolTip NewTooltip()
        {
            return new ToolTip()
            {
                ShowAlways = true,
                ReshowDelay= 0,                 
            };            
        }

        private void DeleteAllLicenseBTN_Click(object sender, EventArgs e)
        {
            DeleteAllLicenses();
        }
        
        private void DeleteSelectedBTN_Click(object sender, EventArgs e)
        {
            DeleteSelectedLicense();
        }

        private void LicenseDetailsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (licenseList.SelectedItems.Count > 0)
            {
                DicomLicense l = licenseList.SelectedItems[0].Tag as DicomLicense;
                MessageBox.Show(GetLicenseDetails(l), LicenseDetailText);
            }
        }

        private void DeleteLicenseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DeleteSelectedLicense();
        }

        private void DeActivateAllLicensesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DeleteAllLicenses();
        }

        private void DeleteSelectedLicense()
        {
            if (licenseList.SelectedItems.Count > 0)
            {
                DicomLicense license = licenseList.SelectedItems[0].Tag as DicomLicense;
                if (MessageBox.Show(DeleteMessage, DeleteCaption, MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
                {
                    license.Delete();
                    FillBox();
                }
            }
        }

        private void DeleteAllLicenses()
        {
            if (licenseList.Items.Count > 0)
            {
                UserInfoPrompt dlg = new UserInfoPrompt();
                if (dlg.ShowDialog() == DialogResult.OK 
                    && !string.IsNullOrEmpty(dlg.CustomerInfo1) 
                    && !string.IsNullOrEmpty(dlg.CustomerInfo2))
                {
                    string res = DicomLicense.DeactivateOnline(dlg.CustomerInfo1, dlg.CustomerInfo2);
                    MessageBox.Show(res, DeactivationResultText, MessageBoxButtons.OK);
                    FillBox();
                }
            }
        }

        private string GetLicenseDetails(DicomLicense license)
        {

            return $@"{AllowedMessageText} {license.AllowedFunctions}{Environment.NewLine}
                    {AuthorisationIDText} {license.AuthorisationID}{Environment.NewLine}
                    {ExpiryDateText} {license.Expiry:D}{Environment.NewLine}
                    {IsRamLicenseText} {license.IsRamLicense}{Environment.NewLine}
                    {LicenseIdText} {license.LicenseID}{Environment.NewLine}
                    {LicenseTokenText} {license.LicenseToken}{Environment.NewLine}
                    {LicenseTypeText} {license.LicenseType}{Environment.NewLine}";
        }
        
    }
}